from twisted.internet.defer import inlineCallbacks, returnValue

from ensemble.providers.common.base import MachineProviderBase

from .cobbler import CobblerClient
from .files import FileStorage
from .launch import OrchestraLaunchMachine
from .machine import machine_from_dict


class MachineProvider(MachineProviderBase):

    def __init__(self, environment_name, config):
        super(MachineProvider, self).__init__(environment_name, config)
        self.cobbler = CobblerClient(config)

    def get_file_storage(self):
        """Retrieve the provider C{FileStorage} abstraction."""
        if "storage-url" not in self.config:
            return FileStorage(
                "http://%(orchestra-server)s/webdav" % self.config)
        return FileStorage(self.config["storage-url"])

    def start_machine(self, machine_data, master=False):
        """Start a machine in the provider.

        @param machine_data: a dictionary of data to pass along to the newly
           launched machine.

        @param master: if True, machine will initialize the ensemble admin
            and run a provisioning agent.
        """
        return OrchestraLaunchMachine(self, master).run(machine_data)

    @inlineCallbacks
    def get_machines(self, instance_ids=()):
        """List machines running in the provider.

        @param instance_ids: ids of instances you want to get. Leave blank
            to list all machines owned by ensemble.

        @return: a list of EC2ProviderMachines.

        @raise: MachinesNotFound
        """
        instances = yield self.cobbler.describe_systems(*instance_ids)
        returnValue([machine_from_dict(i) for i in instances])
