from cStringIO import StringIO

from twisted.web.client import getPage
from twisted.web.error import Error

from ensemble.errors import FileNotFound


class FileStorage(object):

    def __init__(self, base_url):
        self._base_url = base_url

    def _key_to_url(self, name):
        return "%s/%s" % (self._base_url, name)

    def get(self, name):
        url = self._key_to_url(name)
        d = getPage(url)
        d.addCallback(StringIO)

        def convert_404(failure):
            failure.trap(Error)
            if failure.value.status == "404":
                raise FileNotFound(url)
            return failure
        d.addErrback(convert_404)
        return d

    def put(self, remote_path, file_object):
        url = self._key_to_url(remote_path)
        data = file_object.read()
        d = getPage(url, method="PUT", postdata=data)
        d.addCallback(lambda _: True)

        def accept_204(failure):
            # NOTE: webdav returns 204s when we overwrite
            failure.trap(Error)
            if failure.value.status == "204":
                return True
            return failure
        d.addErrback(accept_204)
        return d
